<?php 

namespace Teknisa\DF\Modules\Entrada\Controllers;

/* Bibliotecas */
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

/* Classes DF */
use Teknisa\DF\Modules\Entrada\Services\Dfe71000SincronizacaoCTEService;

class Dfe71000SincronizacaoCTEController {
    
    protected $sincronizacaoCTEService;
    
    public function __construct(Dfe71000SincronizacaoCTEService $sincronizacaoCTEService) {
        $this->sincronizacaoCTEService = $sincronizacaoCTEService;
        set_time_limit(0);
    }
    
    public function montaConfigCte(Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $row = $requestConditions[0]["value"];
        
        try {
            $content = $this->sincronizacaoCTEService->montaConfigCte($row['CDEMPRESA'], $row['NRORG'], $row['CDINSCESTA'], $row["NRINSJURIEST"]);
            $response->addDataset(new DataSet('config_cte', ['config' => $content]));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function sincronizarCte(Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $config = $requestConditions[0]["value"];
        
        try {
            $sincronizacao = $this->sincronizacaoCTEService->sincronizarCte($config);
            $response->addDataset(new DataSet('sincronizarCte', [$sincronizacao]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function gerarXmlCteManifestacao(Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $ctes = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        try {
            $xml = $this->sincronizacaoCTEService->gerarXmlCteManifestacao($ctes, $config);
            
            if($xml) {
                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('gerarXmlCteManifestacao', [$xml]));
            }
            else {
                throw new \Exception('DOWNLOADXMLCTE_ERRO', 500); 
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function prestacaoServicoDesacordoCte(Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $cte = $requestConditions[0]["value"];
        $config = $requestConditions[1]["value"];

        try {
            $dataset = $this->sincronizacaoCTEService->prestacaoServicoDesacordoCte($cte, $config);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('prestServDesacCte', [$dataset]));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}