<?php 

namespace Teknisa\DF\Modules\Entrada\Services;

/* Bibliotecas */
use NFePHP\NFe\Common\Config;
use NFePHP\CTe\Tools as CTe;

use Doctrine\ORM\EntityManager;
use Geral\Entities\Empresa;
use Geral\Entities\Inscrestad;
use NFePHP\Common\Certificate;
/*use NFePHP\Common\Configure\Configure;
use NFePHP\Common\Exception\RuntimeException;
use NFePHP\Common\Files\FilesFolders;


use Teknisa\Libs\Util\DateUtil;*/
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
/*use Teknisa\Model\Fiscal\Entities\Arqxmlnfe;
use Teknisa\Model\Fiscal\Entities\Notadestinada;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Fiscal\Entities\Lotenfe;
use Teknisa\Model\Geral\Entities\Paramempresa;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;*/

/* Classes DF */
use Teknisa\DF\Modules\Entrada\Factories\Dfe71000SincronizacaoCTEFactory;
/*use Teknisa\DF\Services\TransmissaoNfeService;
use Teknisa\DF\Services\XmlNFeService;*/
use Teknisa\DF\Services\NFe\ReturnNFe;

class Dfe71000SincronizacaoCTEService {
    
    //const CONTADOR_NOVOCODIGO_SINCRONIZACAO    = 'SINCRONIZACAONFE';
    const CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO = 'NOTAMANIFESTACAO';
    
    protected $sincronizacaoCTEFactory;
    protected $fileUploader;
    protected $novoCodigo;
    protected $entityManager;
    protected $environment;
    protected $transmissaoService;
    protected $connectionCustom;
    
    protected $cteTools;
    protected $aResposta = [];
    
    //protected $notas = array();
    //protected $notasEncontradas = [];
    protected $notasDownload;
    protected $quantidadeEventos;
    protected $chaves = [];
    protected $quantidadeNotas;
    protected $importacaoPendente = [];
    protected $sincronizadas;
    protected $notasInexistentes = [];
    
    public function __construct(Dfe71000SincronizacaoCTEFactory $sincronizacaoCTEFactory, $novoCodigo, $entityManager, $environment, $fileServerLink, $connectionCustom) {
        $this->sincronizacaoCTEFactory = $sincronizacaoCTEFactory;
        $this->novoCodigo           = $novoCodigo;
        $this->entityManager        = $entityManager;
        $this->environment          = $environment;
        $this->masks = new Masks();
        $this->fileServerLink       = $fileServerLink;
        $this->connectionCustom = $connectionCustom;
        set_time_limit(0);
    }
    
    public function sincronizarCte($config){
        $config = json_decode(json_encode($config));
        $object_config = $config;
        
    	$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
    	$this->cteTools = $tools;
        
        $cnpj = $object_config->cnpj;
        $existeNotas = true;
        
        $ultNSU = $this->sincronizacaoCTEFactory->maxNrNsu($cnpj, $object_config->tpAmb);
        
        $ultNSU = (!$ultNSU) ? 0 : $ultNSU;
        $ultNSU = str_pad($ultNSU, 15, "0", STR_PAD_LEFT);

        $count  = 0; $this->quantidadeEventos = 0; $this->notasDownload = 0; $this->quantidadeNotas = 0; $this->sincronizadas = 0;
        $this->chaves = []; $this->notas = []; $this->notasInexistentes = []; $this->importacaoPendente = [];
        $notas = []; $erros = []; /*$this->notasEncontradas = [];*/
        $connection = $this->entityManager->getConnection();
        
        do {
            try {
                $this->buscaNotasSefaz($ultNSU, 0);
                $notas = $this->aResposta;
                
                $connection->beginTransaction();
                $this->trataRetornoLoteSincronizacao($cnpj, $notas, $object_config);
                $connection->commit();
            }
            catch(\Exception $e) {
                if($e->getCode() === 137) {
                    if($count == 0) {
                        throw new \Exception($e->getMessage(), $e->getCode()); //Nenhum documento localizado.
                    }
                    $existeNotas = false;
                    break;
                }
                else if($e->getCode() === 656){
                    throw new \Exception($e->getMessage(), $e->getCode()); ////Consumo indevido.
                }
                else if(strpos($e->getMessage(), "Certificate") !== false){
                    $connection->rollback();
                    throw new \Exception("Certificado vencido. ".$e->getMessage());
                }

                $connection->rollback();
                $erros[] = $e->getMessage();
            }
            
            if(isset($notas["ultNSU"]) && isset($notas["maxNSU"]) && isset($notas["cStat"])) {
               if($notas["cStat"] == 656 || $count > 11) { // Rejeição 656: Consumo Indevido.
                  $existeNotas = false;
                  break;
               }
                else if($notas["ultNSU"] != $notas["maxNSU"]) {
                    $ultNSU = $notas["ultNSU"];
                    $count++;
                }
                else {
                    $existeNotas = false;
                    break;
                }
            }
            else {
                $existeNotas = false;
                break;
            }
        } while($existeNotas === true);
        
        $this->downloadAutomaticoXML($cnpj, $object_config->tpAmb);
        
        $chavesRetorno = [];
        foreach($this->chaves as $chave) {
            $chavesRetorno[] = $chave;
        }
        
        return array("CHAVES" => $chavesRetorno, "DADOS" => $this->calculaDadosSincronizacao(), "ERROS" => $erros);
    }
    
    public function montaConfigCte($empresa, $organizacao, $inscricao, $cnpj) {
        $objEmpresa = $this->entityManager->getRepository('Geral\Entities\Empresa')->findOneBy(['cdempresa' => $empresa, 'nrorg' => $organizacao]);
        
        $cnpj = $this->masks->removeMascaraInscricao($cnpj);
        
        $razaosocial = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($objEmpresa->getNmrazsocempr())));
        $nomefantasia = $razaosocial;
        
        if(!empty($inscricao)) {
            if(strpos("ISENTO",strtoupper($inscricao)) || strpos("ISENTA", strtoupper($inscricao))){
                $ie = "ISENTO";
            }
            else{
                $ie = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($inscricao)));
            }
            $inscrestad = self::getInscricaoEstadual($empresa, $organizacao, $inscricao, $cnpj);
        }
        else{
            $inscrestad = $this->entityManager->getRepository('Geral\Entities\Inscrestad')->findBy(['cdempresa' => $empresa, 'nrinsjuriest' => $cnpj, 'nrorg' => $organizacao]);

            if(empty($inscrestad)) throw new \Exception('Inscrição Estadual não Encontrada');
            else{
                $ieComCertificados = array_filter($inscrestad, function($certRow){
                    return !empty($this->sincronizacaoCTEFactory->getCertificado($certRow->getCdinscesta(), $certRow->getNrinsjuriest()));
    		    });
    		    
    		    if(empty($ieComCertificados)) throw new \Exception('Certificado Digital não Encontrado');
    		    else{
    		        $inscrestad = $ieComCertificados[0];
    		        $ie = $inscrestad->getCdinscesta();
    		    }
            }
        }
        $tpAmb = (int) $inscrestad->getIdtpambientenfe(); // 1-Produção 2-Homologação
        $siglaUF = $objEmpresa->getSgestado();
        $config = [
            "tpAmb"       => $tpAmb, //REMOVERRRRRRRR!!!!! 1
            "razaosocial" => $nomefantasia,
            "siglaUF"     => $siglaUF,
            "cnpj"        => $cnpj,
            "schemes"     => "PL_CTe_300a",
            "versao"      => "3.00",
            "ie"          => $ie,
            "cdempresa"   => $empresa,
            "nrorg"       => $this->environment->getNrOrgTrab()];
        
        $resp = Config::validate(json_encode($config));
        return $resp;
    }
     /**
     * Retorna inscricao estadual da empresa
     * @param $empresa
     * @param $cdinscestad
     * @param $organizacao
     * @return Inscrestad
     * @throws \Exception
     */
    private function getInscricaoEstadual($empresa, $organizacao, $inscricao, $cnpj){
        $inscrestad = $this->entityManager->getRepository('Geral\Entities\Inscrestad')->findOneBy(
                        ['cdempresa' => $empresa, 'nrinsjuriest' => $cnpj, 'nrorg' => $organizacao, 'cdinscesta'=> $inscricao]);
        
        if (!($inscrestad instanceof Inscrestad)) throw new \Exception('Inscrição Estadual não Encontrada');

        return $inscrestad;
    }
    
    private function downloadAutomaticoXML($cnpj, $tpAmb) {
        $cteDownload = $this->sincronizacaoCTEFactory->getNotasDownloadAutomatico($cnpj, $tpAmb);
        
		foreach($cteDownload as $notaCte) {
		    $notaCte = $notaCte['DSARQXMLNSUCTE'];
		    
		    $xml                    = new \DOMDocument();
            $xml->preservWhiteSpace = false; //elimina espacos em branco
            $xml->formatOutput      = false;
            $xml->loadXML($notaCte);
            $protCTe = $xml->getElementsByTagName('protCTe')->item(0)->nodeValue;
            $chaveCte = $xml->getElementsByTagName('chCTe')->item(0)->nodeValue;
            
    		if($protCTe && $chaveCte) {
			    try {
    			    if($this->environment->getNrOrgTrab() != 1) {
		                if(!isset($this->sincronizacaoCTEFactory->existeNotaRelac($chaveCte)[0])) {
            			    $this->sincronizacaoCTEFactory->zGravaFile($tpAmb, $chaveCte.'.xml', $notaCte, 'importar/', $chaveCte);
        			        $this->notasDownload++;
		                }
        			}
        			else {
        			    $diretorioIntegracao = $this->sincronizacaoCTEFactory->getDiretorioIntegracao();
        			    
    			        if(isset($diretorioIntegracao[0])) {
    			            if($diretorioIntegracao[0]["DSDIRIMPARQXML"] != null) {
    			                if(!is_file($diretorioIntegracao[0]["DSDIRIMPARQXML"]."/$chaveCte.xml")) {
        			                file_put_contents($diretorioIntegracao[0]["DSDIRIMPARQXML"]."/$chaveCte.xml", $notaCte);
        			                $this->notasDownload++;
    			                }
    			            }
    			        }
        			}
			    } catch(\Exception $e) {
		        //var_dump('catch downloadAutomaticoXML', $e->getMessage());
		        }
			}
		}
    }

    public function preencheTabelasSincManifestacao($sincronizacao, $config){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($sincronizacao['aDoc'] as $doc) {
                $nsu = $doc['NSU'];
                $ultNSU = $sincronizacao['ultNSU'];
                $xml                    = new \DOMDocument();
                $xml->preservWhiteSpace = false; //elimina espacos em branco
                $xml->formatOutput      = false;
                $xml->loadXML($doc['doc']);
                $arrayXml = $this->converteXmlParaArray($xml->saveXML());
                
                $chave = $xml->getElementsByTagName('chCTe')->item(0)->nodeValue;
                $cStat = $xml->getElementsByTagName('cStat')->item(0)->nodeValue;
                
                if(!empty($chave)) {
                    if(strtolower(substr($doc['schema'], 0, 7)) == strtolower('procCTe') ||
                       strtolower(substr($doc['schema'], 0, 9)) == strtolower('cteOSProc')) {
                        
                        /*if(!isset($this->notasEncontradas[$chave])) {
                	        $this->notasEncontradas[$chave] = null;
                	    }*/
                	    $this->preencheNotaCte($config, $arrayXml, $chave, $nsu, $doc['schema']);
                	    $this->pesquisaCteAceitos($chave);
                    }
                    else if(strtolower(substr($doc['schema'], 0, 13)) == strtolower('procEventoCTe')) {
                        $this->preencheNotaCte($config, $arrayXml, $chave, $nsu, 'procEventoCTe');
                        $this->verificaEventoCte($xml);
                    }
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function preencheNotaCte($config, $arrayXml, $chave, $nsu, $schema){
        $existeNotaCte = $this->sincronizacaoCTEFactory->existeNotaCte($nsu, $config->cnpj, $config->tpAmb);
        $dadosCte = null;
        
             if(strtolower(substr($schema, 0, 7)) == strtolower('procCTe'))   $dadosCte = $this->preparaDadosProcCTe($chave, $arrayXml, $nsu, $config->cnpj);
        else if(strtolower(substr($schema, 0, 9)) == strtolower('cteOSProc')) $dadosCte = $this->preparaDadosCTeOSProc($chave, $arrayXml, $nsu, $config->cnpj);
        else if($schema == 'procEventoCTe')                                   $dadosCte = $this->preparaDadosProcEventoCTe($chave, $arrayXml, $nsu, $config->cnpj);
        
        if(empty($existeNotaCte)){
            $this->atualizaNSUFaltante($config->cnpj, $config->tpAmb, $nsu);
            
            if(!empty($dadosCte)) {
                $this->sincronizacaoCTEFactory->incluirNotaCte($dadosCte);
            }
        }
        else if(!empty($dadosCte)){
            $this->sincronizacaoCTEFactory->atualizaNotaCte($dadosCte);
        }
    }
    
    private function preparaDadosProcCTe($nracessocte, $arrayXml, $nsu, $cnpj) {
        $CTe = $arrayXml['CTe'];
        $infCte = $CTe['infCte'];
        $ide = $infCte['ide'];
        $emit = $infCte['emit'];
        $rem   = isset($infCte['rem']) ? $infCte['rem'] : null;
        $exped = isset($infCte['exped']) ? $infCte['exped'] : null;
        $receb = isset($infCte['receb']) ? $infCte['receb'] : null;
        $dest = isset($infCte['dest']) ? $infCte['dest'] : null;
        
        $idtpambiente = $ide['tpAmb'];
        $dtEmissao = date('d/m/Y', strtotime(substr($ide["dhEmi"], 0, 19)));
        $vrtotalprest = $infCte['vPrest']['vTPrest'];
        $vrreceber = $infCte['vPrest']['vRec'];
        $cdstatus = $arrayXml['protCTe']['infProt']['cStat'];
        $dsstatus = $arrayXml['protCTe']['infProt']['xMotivo'];
        $nrinsjurreme = null; $nmrazsocreme = null; $nrinsjurexpe = null; $nmrazsocexpe = null; $nrinsjurrece = null;
        $nmrazsocrece = null; $nrinsjurdest = null; $nmrazsocdest = null;
        
        /*EMITENTE*/
            $nrinsjuremit = $emit['CNPJ'];
            $nmrazsocemit = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($emit['xNome'])));
        /*REMETENTE*/
        if(isset($rem)) {
            $nrinsjurreme = isset($rem['CNPJ']) ? $rem['CNPJ'] : $rem['CPF'];
            $nmrazsocreme = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($rem['xNome'])));
        }
        /*EXPEDIDOR*/
        if(isset($exped)) {
            $nrinsjurexpe = isset($exped['CNPJ']) ? $exped['CNPJ'] : $exped['CPF'];
            $nmrazsocexpe = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($exped['xNome'])));
        }
        /*RECEBEDOR*/
        if(isset($receb)) {
            $nrinsjurrece = isset($receb['CNPJ']) ? $receb['CNPJ'] : $receb['CPF'];
            $nmrazsocrece = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($receb['xNome'])));
        }
        /*DESTINATÁRIO*/
        if(isset($dest)) {
            $nrinsjurdest = isset($dest['CNPJ']) ? $dest['CNPJ'] : $dest['CPF'];
            $nmrazsocdest = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($dest['xNome'])));
        }
        /*TOMADOR*/
        if(isset($ide['toma3'])) {
            $idtomador = $ide['toma3']['toma'];
            
            switch($ide['toma3']['toma']) {
                case 0:
                    $nrinsjurtoma = $nrinsjurreme;
                    $nmrazsoctoma = $nmrazsocreme;
                    break;
                case 1:
                    $nrinsjurtoma = $nrinsjurexpe;
                    $nmrazsoctoma = $nmrazsocexpe;
                    break;
                case 2:
                    $nrinsjurtoma = $nrinsjurrece;
                    $nmrazsoctoma = $nmrazsocrece;
                    break;
                case 3:
                    $nrinsjurtoma = $nrinsjurdest;
                    $nmrazsoctoma = $nmrazsocdest;
                    break;
            }
        } else {
            $idtomador = 4;
            $nrinsjurtoma = isset($ide['toma4']['CNPJ']) ? $ide['toma4']['CNPJ'] : $ide['toma4']['CPF'];
            $nmrazsoctoma = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($ide['toma4']['xNome'])));
        }
        
        return ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                "DTEMISSAO"    => $dtEmissao,    "VRTOTALPREST" => $vrtotalprest, "VRRECEBER"    => $vrreceber,    "CDSTATUS"     => $cdstatus,
                "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => $nrinsjuremit, "NMRAZSOCEMIT" => $nmrazsocemit, "NRINSJURREME" => $nrinsjurreme,
                "NMRAZSOCREME" => $nmrazsocreme, "NRINSJUREXPE" => $nrinsjurexpe, "NMRAZSOCEXPE" => $nmrazsocexpe, "NRINSJURRECE" => $nrinsjurrece,
                "NMRAZSOCRECE" => $nmrazsocrece, "NRINSJURDEST" => $nrinsjurdest, "NMRAZSOCDEST" => $nmrazsocdest, "IDTOMADOR"    => $idtomador,
                "NRINSJURTOMA" => $nrinsjurtoma, "NMRAZSOCTOMA" => $nmrazsoctoma];
    }
    
    private function preparaDadosCTeOSProc($nracessocte, $arrayXml, $nsu, $cnpj) {
        $CTeOS = $arrayXml['CTeOS'];
        $infCte = $CTeOS['infCte'];
        $ide = $infCte['ide'];
        $emit = $infCte['emit'];
        $toma = isset($infCte['toma']) ? $infCte['toma'] : null;
        
        $idtpambiente = $ide['tpAmb'];
        $dtEmissao = date('d/m/Y', strtotime(substr($ide["dhEmi"], 0, 19)));
        $vrtotalprest = $infCte['vPrest']['vTPrest'];
        $vrreceber = $infCte['vPrest']['vRec'];
        $cdstatus = $arrayXml['protCTe']['infProt']['cStat'];
        $dsstatus = $arrayXml['protCTe']['infProt']['xMotivo'];
        
        /*EMITENTE*/
        $nrinsjuremit = $emit['CNPJ'];
        $nmrazsocemit = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($emit['xNome'])));
        /*REMETENTE*/
        $nrinsjurreme = null; $nmrazsocreme = null;
        /*EXPEDIDOR*/
        $nrinsjurexpe = null; $nmrazsocexpe = null;
        /*RECEBEDOR*/
        $nrinsjurrece = null; $nmrazsocrece = null;
        /*DESTINATÁRIO*/
        $nrinsjurdest = null; $nmrazsocdest = null;
        /*TOMADOR*/
        $idtomador = null; $nrinsjurtoma = null; $nmrazsoctoma = null;
        if(isset($toma)) {
            $nrinsjurtoma = isset($ide['toma']['CNPJ']) ? $ide['toma']['CNPJ'] : $ide['toma']['CPF'];
            $nmrazsoctoma = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($ide['toma']['xNome'])));
        }
        
        return ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                "DTEMISSAO"    => $dtEmissao,    "VRTOTALPREST" => $vrtotalprest, "VRRECEBER"    => $vrreceber,    "CDSTATUS"     => $cdstatus,
                "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => $nrinsjuremit, "NMRAZSOCEMIT" => $nmrazsocemit, "NRINSJURREME" => $nrinsjurreme,
                "NMRAZSOCREME" => $nmrazsocreme, "NRINSJUREXPE" => $nrinsjurexpe, "NMRAZSOCEXPE" => $nmrazsocexpe, "NRINSJURRECE" => $nrinsjurrece,
                "NMRAZSOCRECE" => $nmrazsocrece, "NRINSJURDEST" => $nrinsjurdest, "NMRAZSOCDEST" => $nmrazsocdest, "IDTOMADOR"    => $idtomador,
                "NRINSJURTOMA" => $nrinsjurtoma, "NMRAZSOCTOMA" => $nmrazsoctoma];
    }
    
    private function preparaDadosProcEventoCTe($nracessocte, $arrayXml, $nsu, $cnpj) {
        $cteAceitos = $this->sincronizacaoCTEFactory->pesquisaCteAceitos($nracessocte);
        
        $idtpambiente = $arrayXml['retEventoCTe']['infEvento']['tpAmb'];
        $cdstatus = $arrayXml['retEventoCTe']['infEvento']['cStat'];
        $dsstatus = $arrayXml['retEventoCTe']['infEvento']['xMotivo'];
            
        if($cteAceitos) {
            $dtEmissao = $cteAceitos['DTEMISSAO'];
            $vrtotalprest = $cteAceitos['VRTOTALPREST'];
            $vrreceber = $cteAceitos['VRRECEBER'];
            
            /*EMITENTE*/
            $nrinsjuremit = $cteAceitos['NRINSJUREMIT'];
            $nmrazsocemit = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCEMIT'])));
            /*REMETENTE*/
            $nrinsjurreme = $cteAceitos['NRINSJURREME'];
            $nmrazsocreme = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCREME'])));
            /*EXPEDIDOR*/
            $nrinsjurexpe = $cteAceitos['NRINSJUREXPE'];
            $nmrazsocexpe = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCEXPE'])));
            /*RECEBEDOR*/
            $nrinsjurrece = $cteAceitos['NRINSJURRECE'];
            $nmrazsocrece = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCRECE'])));
            /*DESTINATÁRIO*/
            $nrinsjurdest = $cteAceitos['NRINSJURDEST'];
            $nmrazsocdest = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCDEST'])));
            /*TOMADOR*/
            $idtomador = $cteAceitos['IDTOMADOR'];
            $nrinsjurtoma = $cteAceitos['NRINSJURTOMA'];
            $nmrazsoctoma = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCTOMA'])));
            
            return ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                    "DTEMISSAO"    => $dtEmissao,    "VRTOTALPREST" => $vrtotalprest, "VRRECEBER"    => $vrreceber,    "CDSTATUS"     => $cdstatus,
                    "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => $nrinsjuremit, "NMRAZSOCEMIT" => $nmrazsocemit, "NRINSJURREME" => $nrinsjurreme,
                    "NMRAZSOCREME" => $nmrazsocreme, "NRINSJUREXPE" => $nrinsjurexpe, "NMRAZSOCEXPE" => $nmrazsocexpe, "NRINSJURRECE" => $nrinsjurrece,
                    "NMRAZSOCRECE" => $nmrazsocrece, "NRINSJURDEST" => $nrinsjurdest, "NMRAZSOCDEST" => $nmrazsocdest, "IDTOMADOR"    => $idtomador,
                    "NRINSJURTOMA" => $nrinsjurtoma, "NMRAZSOCTOMA" => $nmrazsoctoma];
        }
        else {
            return ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                    "DTEMISSAO"    => null,          "VRTOTALPREST" => null,          "VRRECEBER"    => null,          "CDSTATUS"     => $cdstatus,
                    "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => null,          "NMRAZSOCEMIT" => null,          "NRINSJURREME" => null,
                    "NMRAZSOCREME" => null,          "NRINSJUREXPE" => null,          "NMRAZSOCEXPE" => null,          "NRINSJURRECE" => null,
                    "NMRAZSOCRECE" => null,          "NRINSJURDEST" => null,          "NMRAZSOCDEST" => null,          "IDTOMADOR"    => null,
                    "NRINSJURTOMA" => null,          "NMRAZSOCTOMA" => null];
        }
    }
    
    private function pesquisaCteAceitos($nracessocte) {
        $cteAceitos = $this->sincronizacaoCTEFactory->pesquisaCteAceitos($nracessocte);
        if($cteAceitos) $this->sincronizacaoCTEFactory->atualizaCteAceitos($cteAceitos);
    }
    
    private function verificaEventoCte($xml) {
        
        $tpEvento = $xml->getElementsByTagName('tpEvento')->item(0)->nodeValue;
        
        if($tpEvento) {
            if($tpEvento === '110110' || $tpEvento === '110111') $this->quantidadeEventos++;
        }
    }
    
    /*private function preencheEventoNotadestinada($notadestinada, $config, $xml, $chave, $docEvento, $existeTpEvento){
        $nota = $this->converteXmlParaArray($xml->saveXML());
        $correcao = false;
        $dsOperacao = '';
        if($existeTpEvento) $tpEvento = $xml->getElementsByTagName('tpEvento')->item(0)->nodeValue;
        else $tpEvento = '100';
        if($notadestinada) {
            if($tpEvento === '110111') {
                $idTpEvento  = '1';
                $idStatusNfe = '3'; 
                $dsOperacao  = "Nota fiscal cancelada pelo destinatário.";        
                $dtRecbto = $nota["retEvento"]["infEvento"]["dhRegEvento"];
                $dtRecbto = date('d/m/Y H:i:s', strtotime(substr($dtRecbto, 0, 19)));
                $existeNotaDestinada = $this->sincronizacaoCTEFactory->existeNotaDestinada($chave);
                
                if(isset($existeNotaDestinada[0])){
                    $this->sincronizacaoCTEFactory->atualizaNotaCte($idStatusNfe, '', $dtRecbto, $chave);
                }
                else{
                    $nrProxNotadestinada = $this->novoCodigo->getCodeByTableName(self::CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO, $config->nrorg);
                    $this->sincronizacaoCTEFactory->incluirNotadestinada($nrProxNotadestinada, '', $chave, '', '',
                                    '', '', '', '', '', '', $idStatusNfe,
                                    '', '', null, null, null, null, null, '', null, '', '', '', '', '', $this->environment->getNrOrgTrab());    
                } 
                $this->quantidadeEventos++;
            }
            else if($tpEvento === '110110') {
                $idTpEvento  = '2';
                $idStatusNfe = '0';
                $dsOperacao  = "Evento de carta de correção registrado para a nota fiscal ".$nota["evento"]["infEvento"]["detEvento"]["xCorrecao"];
                $dtHrEvento  = date('d/m/Y', strtotime(substr($nota["retEvento"]["infEvento"]["dhRegEvento"], 0, 19)));
                $dsCorrecaoNfe = $nota["evento"]["infEvento"]["detEvento"]["xCorrecao"];
                $nrSeqEvento = $this->sincronizacaoCTEFactory->getNrSeqEvento($chave)[0]["NRSEQEVENTO"]; 
                $correcao = true;
                $this->sincronizacaoCTEFactory->atualizaEventoNotadestinada(null, null, null, 2, $dtHrEvento, $nrSeqEvento, $dsCorrecaoNfe ,$chave);
                $this->quantidadeEventos++;
            }
            else if($tpEvento == '210200' 
                || $tpEvento  == '210210'
                || $tpEvento  == '210220'
                || $tpEvento  == '210240') {
                
                $idTpEvento  = '3';
                $idStatusNfe = '1';
            }
            else {
                $idStatusNfe = '0'; 
                $idTpEvento  = '0';
            }
            
            $sequencial = $this->sincronizacaoCTEFactory->getSequencialHistoricoXML($chave)[0]["NRSEQHISTOR"];
            if($sequencial === null) $sequencial = 1;
            else $sequencial += 1;
            $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
            
            $notaSincronizada = $this->sincronizacaoCTEFactory->getNotaSincronizada($chave);
            
            $cdFilial   = isset($notaSincronizada[0]) ? $notaSincronizada[0]["CDFILIAL"] : '';
            $nrLanctonf = isset($notaSincronizada[0]) ? $notaSincronizada[0]["NRLANCTONF"] : '';
            $idStatus   = isset($notaSincronizada[0]) ? 'S' : 'N';
            
            if($correcao === true) {
                $nProt      = $nota["retEvento"]["infEvento"]["nProt"];
            }
            else {
                try {
                    $nProt      = $nota["evento"]["infEvento"]["detEvento"]["nProt"];
                }
                catch(\Exception $e) {
                    $nProt      = $nota["retEvento"]["infEvento"]["nProt"];
                }
            }
            
            $tipoOper   = isset($this->sincronizacaoCTEFactory->buscaHistoricoXML($chave)[0]) ? 1 : 2;
            
            $this->sincronizacaoCTEFactory->insereHistoricoXML($chave, $sequencial, date('d/m/Y H:i:s'), $tipoOper, $idStatus, $nProt, $dsOperacao, $cdFilial, $nrLanctonf, $this->environment->getCdOperador(), $this->environment->getNrOrg());
        }
        // TODO 
        // Buscar nota destinada por chave de acesso caso não exista no sistema
    }*/

    public function manifestarNfe($notasfiscais, $config, $evento = '', $justificativa = '')  {
        $config = json_encode($config);
        $object_config = json_decode($config);
    	if(!isset($this->cteTools)) {
    		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, $config);
    		$this->cteTools = $tools;
        }
        
        $array_erros = array();
        foreach ($notasfiscais as $nota) {
            $erro = $this->manifestaNfe($nota, $evento, $justificativa, $config);
            if($erro) {
                $erro['NRLANCTONF'] = $nota['NRLANCTONF'];
                $erro['NRNOTAFISC'] = $nota['NRNOTAFISC'];
                array_push($array_erros, $erro);
            }
        }
        if(empty($array_erros)) return array("SUCESSO" => "Nota(s) manifestada(s) com sucesso.");
        else return array("ERRO" => $array_erros);
    }

    public function gerarXmlCteManifestacao($ctes, $config){
        $object_config = json_decode(json_encode($config));
        
    	if(!isset($this->cteTools)) {
    		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
    		$this->cteTools = $tools;
        }
        
        $existeCTe = false;
        $dir = sys_get_temp_dir();
        $tpAmb = $object_config->tpAmb;
        $zipname = $dir.DIRECTORY_SEPARATOR.uniqid().'XMLs.zip';
        $zip = new \ZipArchive();

        if($zip->open($zipname, \ZipArchive::CREATE) === true){
            foreach ($ctes as $cte) {
                $chave = $cte['NRACESSOCTE'];
                
                $xmlCTe = $this->sincronizacaoCTEFactory->getXMLCTe($object_config->cnpj, $tpAmb, $cte['NRNSU']);
                
                if(!empty($xmlCTe)) {
                    file_put_contents($dir.DIRECTORY_SEPARATOR.$chave.'.xml', $xmlCTe['DSARQXMLNSUCTE']);
                    $xmlZip = $dir.DIRECTORY_SEPARATOR.$chave.'.xml';
                }
                else{
                    $xmlZip = $this->downloadCTe($cte['NRNSU'], $dir, $chave, $object_config);
                }
                
                if(!empty($xmlZip)) {
                    $existeCTe = true;
                    $zip->addFromString('CTe-'.$chave.'.xml' , file_get_contents($xmlZip));
                    unlink($xmlZip);
                }
            }
            $zip->close();
        }
        
        if(!$existeCTe) return false;
        else            return $zipname;
    }
    
    private function buscaNotasSefaz($ultNSU, $numNSU){
        $this->aResposta = [];
        $resp = $this->cteTools->sefazDistDFe($ultNSU, $numNSU, 'AN');
        
        $this->aResposta = ReturnNFe::readReturnSefaz('NfeDistribuicaoDFe', $resp);
        
        return $this->aResposta;
    }

    private function manifestaNfe($nota, $tpEvento, $xJust, $config){
        $notaEntidade = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notadestinada')->findOneBy(
                            array('nracessonfe' => $nota['NRACESSONFE']));
        if(($tpEvento == "2" || $tpEvento == "3") && isset($notaEntidade)) {
            return array("CODIGO" => '', "MOTIVO" => "Nota existente no sistema. Não é possivel realizar operações
            do tipo \"Operação não realizada\" ou \"Desconhecimento da operação\".");
        }
        else {
            $this->aResposta = array();
            $chave = $nota['NRACESSONFE'];
            $tpAmb = $this->cteTools->config->tpAmb;
            
            switch($tpEvento) {
                case "1":
                    $tpEvento = "210200";
                    $dsOperacao    = "Operação Confirmada";
                    break;
                
                case "4": 
                    $idStatusManif = "210210";
                    $dsOperacao    = "Ciência da Emissão";
                    break;
                
                case "2":
                    $idStatusManif = "210220";
                    $dsOperacao    = "Operação Desconhecida";
                    break;
                    
                case "3":
                    $idStatusManif = "210240";
                    $dsOperacao    = "Operação Não Realizada";
                    break;
            }
            
            $eventos    = array("sucesso" => array(), "erro" => array());
            $xmlRetorno = $this->cteTools->sefazManifesta($chave, $tpEvento, $xJust, 1, $this->aResposta);
            
            $this->aResposta = ReturnNFe::readReturnSefaz('RecepcaoEvento', $xmlRetorno);
            
            return $this->trataLoteRetorno($this->aResposta, $xmlRetorno, $config, $tpEvento);
        }
        
    }

    private function trataLoteRetorno($loteRetorno, $xmlRetorno, $config, $tpEvento) {
        switch($tpEvento) {
            case "210200":
                $idStatusManif = "1";
                $dsOperacao    = "Operação Confirmada";
                break;
            
            case "210210": 
                $idStatusManif = "4";
                $dsOperacao    = "Ciência da Emissão";
                break;
            
            case "210220":
                $idStatusManif = "2";
                $dsOperacao    = "Operação Desconhecida";
                break;
                
            case "210240":
                $idStatusManif = "3";
                $dsOperacao    = "Operação Não Realizada";
                break;
        }

        // Verifica erro no lote
        if($loteRetorno['cStat'] == '128') {
            // Verifica erro nos eventos
            foreach ($loteRetorno['evento'] as $evento) {
                if($evento['cStat'] == '135' || ($evento['cStat'] == '136' && $evento['xMotivo'] == 'Evento registrado, mas nao vinculado a NF-e')) {
                    $notaDestinada = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notadestinada')->findOneBy(
                            array('nracessonfe' => $evento['chNFe']));
                    if($notaDestinada) {
                        $this->sincronizacaoCTEFactory->atualizaEventoNotadestinadaManifestada($notaDestinada, $evento, $idStatusManif, $xmlRetorno, $this->cteTools->config->cdempresa, $this->cteTools->config->nrorg);
                    }
                    else {
                        $this->sincronizacaoCTEFactory->incluirNotadestinadaManifestada($evento, $config, $idStatusManif, $xmlRetorno, $this->cteTools->config->cdempresa, $this->cteTools->config->nrorg);
                        $this->sincronizacaoCTEFactory->deletaNotaDuplicada($evento["chNFe"], $config->cnpj);
                    }
                    
                    $dadosNf =  $this->sincronizacaoCTEFactory->getDadosNf($evento['chNFe']);
                    if(isset($dadosNf[0])) {
                          if($dadosNf[0]["IDGERADAVIAXML"] == "S") {
                              $this->sincronizacaoCTEFactory->insereHistoricoXML($chave, $sequencial, date('d/m/Y H:i:s'), $tipoOper, $idStatus, $nProt, $dsOperacao, $cdFilial, $nrLanctonf, $this->environment->getCdOperador(), $this->environment->getNrOrg());
                          }
                    }
                }
                else {                    
                    $this->sincronizacaoCTEFactory->atualizaCdStatusManif($evento["chNFe"], $evento["cStat"]);
                    $erro = array($evento['cStat'], $evento['xMotivo'], $tpEvento);
                }
            }
            if(isset($erro)) {
                return array("CODIGO" => $evento['cStat'], "MOTIVO" => $evento['xMotivo']);
            }
        }
        else return array("CODIGO" => $loteRetorno['cStat'], "MOTIVO" => $loteRetorno['xMotivo']);
    }
    
    function formataData($date) {
        $dateDMY  = substr($date, 0, 10);
        $dateHIS  = substr($date, 11, 8);
        $formatedDate = explode('-', $dateDMY);
        $formatedDate = $formatedDate[0].'-'.$formatedDate[1].'-'.$formatedDate[2];
        $formatedDate.= ' '.$dateHIS;
        $formatedDate = strtotime($formatedDate);
        $formatedDate = date("d/m/Y H:i:s", $formatedDate);
        return DateUtil::getDataDeString($formatedDate, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
    }

    private function downloadCTe($nsu, $dir, $chave, $object_config){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->buscaNotasSefaz(0, $nsu);
            $notas = $this->aResposta;
            
            $this->trataRetornoLoteSincronizacao($object_config->cnpj, $notas, $object_config);
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollback();
            throw new \Exception($e->getMessage(), $e->getCode());
        }
        
        if(isset($this->aResposta)) {
            if(!empty($this->aResposta["aDoc"])) {
                file_put_contents($dir.DIRECTORY_SEPARATOR.$chave.'.xml', $this->aResposta["aDoc"][0]["doc"]);
                return $dir.DIRECTORY_SEPARATOR.$chave.'.xml';
            }
        }
        else {
            return false;
        }
    }

    private function recolocarArrobaEmail($config) {
        $config            = json_decode($config);
        $config->aMailConf = json_decode(str_replace("&#64", "@", json_encode($config->aMailConf)));
        $config            = json_encode($config);
        return $config;
    }
    
    /**
    *   Converte uma string Xml para um array.
    *    
    *   @param string $stringXml
    *   return  array - Array do XML
    */
    private function converteXmlParaArray($stringXml) {
        $xml = simplexml_load_string($stringXml);
        $json = json_encode($xml);
        $array = json_decode($json,TRUE);
        return $array;
    }
    
    private function calculaDadosSincronizacao() {
        $dadosSincronizacao = [];
        $dadosSincronizacao["QTNOTAS"]   = "Foram encontradas ".count($this->chaves)." nota(s) de CTE.";
        $dadosSincronizacao["QTEVENTOS"] = "Foram encontrados $this->quantidadeEventos do tipo cancelamento ou carta de correção.";
        $dadosSincronizacao["QTINEXIST"] = "Existem ".count($this->notasInexistentes)." notas fiscais destinadas inexistentes no sistema.";
        $dadosSincronizacao["QTIMPORTP"] = "Existem ".count($this->importacaoPendente)." notas fiscais com importação de XML pendente.";
        $dadosSincronizacao["SINCRONIZ"] = "Foram atualizadas $this->sincronizadas notas fiscais existentes no sistema";
        $dadosSincronizacao["NOTASDOWN"] = "Download de $this->notasDownload arquivos XML realizado com sucesso";
        return $dadosSincronizacao;
    }

	public function getToolsNFe($ie, $cnpj, $config) {
	    
	    /*Função criada para buscar o cnpj vinculada a isncrição estadual, de forma que ele sempre busque o certificado correto*/
	    
	    $cnpjInscEstad = $this->sincronizacaoCTEFactory->getCNPJCertificado($ie, json_decode($config)->cdempresa);
	    $certificado = $this->sincronizacaoCTEFactory->getCertificado($ie, $cnpjInscEstad);
		$certificadoEncontrado = false;
		
		foreach($certificado as $cert) {
		    if($cert["DSDIRCERTIFICADO"]) {
		        $certificadoEncontrado = true;
		        $certify = $cert;
		    }
		}
		
		if($certificadoEncontrado == false) throw new \Exception("Certificado Digital não encontrado.");
		
        $senha = $this->decrypt($certify["CDSENHACERTIF"]);
		$certificado = $certify["DSDIRCERTIFICADO"];
		$certificado = explode('>', $certificado);
        $certificado = file_get_contents($this->fileServerLink.$certificado[1]);
        
        try {
            $certificado = Certificate::readPfx($certificado, $senha);
        }
        catch(\Exception $e) {
            throw new \Exception("Senha do certificado incorreta!");
        }
        $tools = new CTe($config, $certificado);
        
        return $tools;
	}
	
	private function decrypt($text, $salt = 'teknisa') {
	    return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    public function getDefaultFilterManif($row) {
        $cdfilial = ($this->environment->getCdFilial()) ? $this->environment->getCdFilial() : $row["CDFILIAL"];
        $defaultFilter = $this->sincronizacaoCTEFactory->getDefaultFilterManif($cdfilial);
        $defaultFilter["DTEMISSAO"] = date("d/m/Y");
        return $defaultFilter;
    }
    
    public function nvl(&$var, $default = ""){
        return isset($var) ? $var
                           : $default;
    }
    
    private function trataRetornoLoteSincronizacao($cnpj, $notas, $object_config) {
        $cStat = $notas["cStat"];
        $xMotivo = $notas["xMotivo"];
        
        if($notas["cStat"] == 138) { //Documento localizado.
            
            //$this->notas = array_merge($this->notas, $notas["aDoc"]);
            
            if(isset($notas["aDoc"])) {
                $this->preencheTabelasSincManifestacao($notas, $object_config);
                
                foreach($notas["aDoc"] as $notaCte) {
                    
                    $arrayXml = $this->converteXMLParaArray($notaCte['doc']);
                    $nsu = $notaCte['NSU'];
                    $tipoNota = false; $tipoEvento = false;
                    
                    //procCTe e cteOSProc
                    if(strtolower(substr($notaCte["schema"], 0, 7)) == strtolower('procCTe') ||
                       strtolower(substr($notaCte["schema"], 0, 9)) == strtolower('cteOSProc')) {
                        
                        $protCTe = $arrayXml['protCTe'];
                        $infProt = $protCTe['infProt'];
                        $chave = $infProt['chCTe'];
                        $tipoNota = true;
                    	
                    	if($infProt['cStat'] !== 110) {
                    	    $existeNotaCte = $this->sincronizacaoCTEFactory->existeNotaCte($nsu, $cnpj, $infProt['tpAmb']);
                    	    
                    	    if($existeNotaCte) {
                    	        $this->verificaXmlCte($cnpj, $infProt['tpAmb'], $nsu, $notaCte['doc'], null, null);
                    	    }
                    	}
                    	
                    	if(!isset($this->chaves[$chave])) {
            			    $this->chaves[$chave] = $chave; //Apenas ctes do tipo "procCTe" ou "cteOSProc" são considerados para retorno.
            			}
                    }
                    //procEventoCTe
                    else if(strtolower(substr($notaCte["schema"], 0, 13)) == strtolower('procEventoCTe')) {
                    	$retEventoCTe = $arrayXml['retEventoCTe'];
                    	$infEvento = $retEventoCTe['infEvento'];
                    	$chave = $infEvento['chCTe'];
                    	$tipoEvento = true;
                    	
                    	if($infEvento['cStat'] !== 110) {
                    	    $existeNotaCte = $this->sincronizacaoCTEFactory->existeNotaCte($nsu, $cnpj, $infEvento['tpAmb']);
                    	    
                    	    if($existeNotaCte) {
                        	    $this->verificaXmlCte($cnpj, $infEvento['tpAmb'], $nsu, $notaCte['doc'], $infEvento['tpEvento'], $infEvento['xEvento']);
                    	    }
                    	}
                    }
                    
                    if($tipoNota === true || $tipoEvento === true) {
                        $notaSincronizada = $this->sincronizacaoCTEFactory->getNotaSincronizada($chave);
                        $somenteXML = $this->sincronizacaoCTEFactory->somenteXML($chave);
                        
                        if($tipoNota) {
                    	    if(!isset($somenteXML[0]) && !isset($notaSincronizada[0])) {
                			    if(!isset($this->notasInexistentes[$chave])) {
                                    $this->notasInexistentes[$chave] = null;
                    	        }
                			}
                    	}
                        if(isset($notaSincronizada[0])) {
                        	$this->sincronizadas++;
                        }
                        else {
                        	if(isset($somenteXML[0])) {
                            	if(!isset($this->importacaoPendente[$chave])) {
                                    $this->importacaoPendente[$chave] = null;
                                }
                        	}
                        }
                    }
                }
            }
        }
        else if($notas["cStat"] == 137) throw new \Exception($notas['xMotivo'], 137); //Nenhum documento localizado.
        else if($notas["cStat"] == 656) throw new \Exception($notas['xMotivo'], 656); //Consumo indevido.
    }
    
    /*private function insereNsuNfe($cnpj, $tpAmb, $ultNsu, $maxNsu){
        $maxNrNsu = $this->sincronizacaoCTEFactory->maxNrNsu($cnpj, $tpAmb);
        $data = $this->sincronizacaoCTEFactory->getDataHoraCerta();

        if($maxNrNsu == 0) {
            for($i = ($ultNsu - 100); $i <= $maxNsu; $i++) {
                if($i >= 0) {
                    $nsu = str_pad($i, 15, "0", STR_PAD_LEFT);
                    $nsuExiste = $this->sincronizacaoCTEFactory->consultaNsu($cnpj, $tpAmb, $nsu);
    
                    if(empty($nsuExiste)) {
                        $this->sincronizacaoCTEFactory->insereNsuNfe($cnpj, $tpAmb, $nsu, $data["DATAHORACERTA"], $data["DATAHORACERTA"], '', '', '', '', 'N', '', '', '', '', '', $this->environment->getNrOrgTrab());
                    }
                }
            }
        }
        else {
            for($i = $maxNrNsu; $i <= $maxNsu; $i++) {
                $nsu = str_pad($i, 15, "0", STR_PAD_LEFT);
                $nsuExiste = $this->sincronizacaoCTEFactory->consultaNsu($cnpj, $tpAmb, $nsu);
                if(empty($nsuExiste)) {
                    $this->sincronizacaoCTEFactory->insereNsuNfe($cnpj, $tpAmb, $nsu, $data["DATAHORACERTA"], $data["DATAHORACERTA"], '', '', '', '', 'N', '', '', '', '', '', $this->environment->getNrOrgTrab());
                }
            }
        }

    }*/
    
    private function verificaXmlCte($cnpj, $tpamb, $nrnsu, $xml, $cdservico, $dsservico) {
        
        if(!$this->sincronizacaoCTEFactory->existeXmlNsuCte($cnpj, $tpamb, $nrnsu)) {
            $this->sincronizacaoCTEFactory->insereXmlNsuCte($cnpj, $tpamb, $nrnsu, $xml, $cdservico, $dsservico, $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
        }
    }
    
    /*private function atualizaNsuNfe($dtinsnrnsu, $dtupdnrnsu, $dtrecebto, $nracessonfe, $cdstatus, $dsmotivo, $dsservico, $idconsultado, $nrprotocolo, $nrinsjuriest, $tpamb, $nrnsu, $idstatusnfe, $cdservico, $nrseqservico, $xml) {
        $dtrecebto = date('d/m/Y H:i:s', strtotime(substr($dtrecebto, 0, 19)));
        $this->sincronizacaoCTEFactory->atualizaNsuNfe($dtinsnrnsu, $dtupdnrnsu, $nracessonfe, $cdstatus, $dsmotivo, $dsservico, $idconsultado, $dtrecebto, $nrprotocolo, $idstatusnfe, $cdservico, $nrseqservico, $nrinsjuriest, $tpamb, $nrnsu);
        
        if($xml) {
            if(!$this->sincronizacaoCTEFactory->existeXmlNsu($nrinsjuriest, $tpamb, $nrnsu)) {
                $this->sincronizacaoCTEFactory->insereXmlNsu($nrinsjuriest, $tpamb, $nrnsu, $xml, $this->environment->getNrOrgTrab());
            }
        }
    }*/
    
    private function atualizaNSUFaltante($cnpj, $tpAmb, $nsu) {
        $maxNrNsu = $this->sincronizacaoCTEFactory->maxNrNsu($cnpj, $tpAmb);
        
        if($maxNrNsu > 0) {
            $prxNrNSU = ltrim($nsu, "0");
            
            for($i = ($maxNrNsu + 1); $i <= ($prxNrNSU - 1); $i++) {
                
                $dadosCte = [
                    "NSU"          => str_pad($i, 15, "0", STR_PAD_LEFT), "DSSTATUS"     => "NSU inexistente no Sistema Teknisa",
                    "CDSTATUS"     => "999", "NRACESSOCTE"  => null,      "NRINSJURPESQ" => $cnpj, "IDTPAMBIENTE" => $tpAmb,
                    "DTEMISSAO"    => null,  "VRTOTALPREST" => null,      "VRRECEBER"    => null,  "NRINSJUREMIT" => null, 
                    "NMRAZSOCEMIT" => null,  "NRINSJURREME" => null,      "NMRAZSOCREME" => null,  "NRINSJUREXPE" => null, 
                    "NMRAZSOCEXPE" => null,  "NRINSJURRECE" => null,      "NMRAZSOCRECE" => null,  "NRINSJURDEST" => null, 
                    "NMRAZSOCDEST" => null,  "IDTOMADOR"    => null,      "NRINSJURTOMA" => null,  "NMRAZSOCTOMA" => null];
                
                $this->sincronizacaoCTEFactory->incluirNotaCte($dadosCte);
            }
        }
    }
    
    public function setAgendamentoManifestacao($nrorg, $useVpd) {
        if($useVpd){
            $newEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
            
            if(empty($newEntityManager)) throw new \Exception('Falha ao gerar o newEntityManager.');
            
            $this->entityManager = $newEntityManager;
            $this->novoCodigo = new NewCode($newEntityManager->getConnection(), 'NOVO_CODIGO_SC');
        }
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
        $this->sincronizacaoCTEFactory->setAgendamentoManifestacao($nrorg, $useVpd, ($useVpd) ? $newEntityManager : null);
    }
    
    public function getManifestacaoFactory() {
        return $this->sincronizacaoCTEFactory;
    }
    
    public function getNewEntityManager() {
        return $this->entityManager;
    }
    
    private function replaceCharsForParserInverted($string)
    {
        $from = [
            "&lt;",
            "&gt;",
            "&amp;",
            "&quot;",
            "&#39;"
        ];
        
        $to = [
            "<",
            ">",
            "&",
            "\"",
            "''"
        ];

        return str_replace($from, $to, $string);
    }
    
    public function prestacaoServicoDesacordoCte($cte, $config) {
        if($cte['NRINSJURPESQ'] !== $cte['NRINSJURTOMA']) throw new \Exception('CTE_RECUSA_SERV_DESAC');
        
        $object_config = json_decode(json_encode($config));
    	if(!isset($this->cteTools)) {
    		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
    		$this->cteTools = $tools;
        }
        
        //$recusaServDesac = $this->Dfe71000SincronizacaoCTEFactory->recusaServDesac($cte['NRINSJURPESQ'], $cte['IDTPAMBIENTE'], $cte['NRNSU'], $cte['NRACESSOCTE']);
        
        
        
        /*$this->aResposta = [];
        $resp = $this->cteTools->sefazManifesta($cte['NRACESSOCTE'], 610110, 'AN');
        
        
        $this->aResposta = ReturnNFe::readReturnSefaz('NfeDistribuicaoDFe', $resp);*/
        
        
        
        
    }
}
